// SPDX-License-Identifier: LicenseRef-AGPL-3.0-only-OpenSSL

package co.vulcanlabs.psremote.common

import co.vulcanlabs.psremote.utils.convert
import com.google.gson.Gson
import com.google.gson.TypeAdapter
import com.google.gson.TypeAdapterFactory
import com.google.gson.reflect.TypeToken
import com.google.gson.stream.JsonReader
import com.google.gson.stream.JsonToken
import com.google.gson.stream.JsonWriter
import java.io.IOException
import java.text.DateFormat
import java.text.ParseException
import java.text.SimpleDateFormat
import java.util.*

class ImprovedDateTypeAdapter : TypeAdapter<Date>() {
    private val enUsFormat: DateFormat = DateFormat.getDateTimeInstance(2, 2, Locale.US)
    private val localFormat: DateFormat = DateFormat.getDateTimeInstance(2, 2, Locale.TRADITIONAL_CHINESE)
    private val iso8601Format: DateFormat = buildIso8601Format()

    @Throws(IOException::class)
    override fun read(`in`: JsonReader): Date? {
        if (`in`.peek() == JsonToken.NULL) {
            `in`.nextNull()
            return null
        }
        return deserializeToDate(`in`.nextString())
    }

    @Synchronized
    private fun deserializeToDate(json: String): Date? {
        try {

            return Date(json.replace(".", "").toLong())
        } catch (e: Exception) {

            try {

                return this.localFormat.parse(json)
            } catch (e1: ParseException) {

                return try {
                    this.enUsFormat.parse(json)
                } catch (e2: ParseException) {

                    try {
                        this.iso8601Format.parse(json)
                    } catch (e3: ParseException) {
                        null
                    }

                }

            }

        }

    }

    @Synchronized
    @Throws(IOException::class)
    override fun write(out: JsonWriter, value: Date?) {
        if (value == null) {
            out.nullValue()
            return
        }
        //        String dateFormatAsString = this.enUsFormat.format(value);
        //        out.value(dateFormatAsString);
        out.value(iso8601Format.format(value))
    }

    companion object {

        val FACTORY: TypeAdapterFactory = object : TypeAdapterFactory {

            override fun <T> create(gson: Gson, typeToken: TypeToken<T>): TypeAdapter<T>? {

                return (if (typeToken.rawType == Date::class.java)
                    ImprovedDateTypeAdapter()
                else
                    null)?.convert<TypeAdapter<T>>()
            }
        }

        fun buildIso8601Format(pattern: String = "yyyy-MM-dd'T'HH:mm:ss'Z'"): DateFormat {
            val iso8601Format = SimpleDateFormat(
                pattern, Locale.US
            )
            iso8601Format.timeZone = TimeZone.getTimeZone("UTC")
            return iso8601Format
        }
    }
}