// SPDX-License-Identifier: LicenseRef-AGPL-3.0-only-OpenSSL

package co.vulcanlabs.psremote.data

import android.app.Application
import android.content.Context
import co.vulcanlabs.psremote.utils.convert
import co.vulcanlabs.psremote.utils.getSharePref

open class BaseSharePreference(val context: Context) {

    constructor(app: Application) : this(app.applicationContext)

    companion object {
        const val PREF_REFERRAL_INFO = "PREF_REFERRAL_INFO"
    }

    fun <T> storeData(key: String, data: T) {
        context.getSharePref().edit().apply {
            when (data) {
                is Int -> putInt(key, data)
                is Long -> putLong(key, data)
                is Boolean -> putBoolean(key, data)
                is String -> putString(key, data)
                is Float -> putFloat(key, data)
                is Set<*> -> putStringSet(key, data.convert())
            }
            apply()
        }
    }

    inline fun <reified T> getData(key: String, default: T) = context.getSharePref().let {
        when (T::class) {
            Int::class -> it.getInt(key, default as Int)
            Long::class -> it.getLong(key, default as Long)
            Boolean::class -> it.getBoolean(key, default as Boolean)
            String::class -> it.getString(key, default as String)
            Float::class -> it.getFloat(key, default as Float)
            Set::class -> it.getStringSet(key, null)
            else -> default
        }
    }?.convert<T>() ?: default

    val isFirstOpen: Boolean
        get() = "VULCAN_FIRST_OPEN".let { key ->
            getData(key, true).apply {
                storeData(key, false)
            }
        }
}