// SPDX-License-Identifier: LicenseRef-AGPL-3.0-only-OpenSSL

package co.vulcanlabs.psremote.data

import co.vulcanlabs.psremote.common.UserAccount
import co.vulcanlabs.psremote.data.local.Preferences
import co.vulcanlabs.psremote.data.network.service.PsnLoginService
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.withContext
import javax.inject.Inject

class UserRepositoryImpl @Inject constructor(
    private val psnLoginService: PsnLoginService,
    private val preferences: Preferences
) : UserRepository {
    override suspend fun loginPsn(code: String): Result<Unit> =
        withContext(Dispatchers.IO) {
            try {
                val token = psnLoginService.requestToken(code = code)
                val psnInfo = psnLoginService.requestAccountInfo(token = token.accessToken)
                preferences.currentUser = UserAccount(psnInfo.userId, psnInfo.onlineId)
                Result.Success(Unit)
            } catch (e: Exception) {
                Result.Failure(e)
            }
        }

    override fun getCurrentUser(): UserAccount? = preferences.currentUser

    override suspend fun logOut(): Result<Unit> =
        withContext(Dispatchers.IO) {
            try {
                preferences.run {
                    currentUser = null
                    playSettingOrientationLockMode = "AUTO_ROTATE"
                    playSettingAlwaysShowController = true
                }
                Result.Success(Unit)
            } catch (e: Exception) {
                Result.Failure(e)
            }
        }
}