// SPDX-License-Identifier: LicenseRef-AGPL-3.0-only-OpenSSL

package co.vulcanlabs.psremote.data.local

import android.content.Context
import androidx.room.*
import co.vulcanlabs.psremote.chiaki.Target
import co.vulcanlabs.psremote.common.*

@Database(
    version = 1,
    entities = [RegisteredHost::class, ManualHost::class]
)
@TypeConverters(Converters::class)
abstract class AppDatabase : RoomDatabase() {
    abstract fun registeredHostDao(): RegisteredHostDao
    abstract fun manualHostDao(): ManualHostDao
//    abstract fun importDao(): ImportDao

    companion object {
        @Volatile
        private var database: AppDatabase? = null
        fun getInstance(context: Context): AppDatabase {
            if (database == null) {
                synchronized(AppDatabase::class.java) {
                    if (database == null) {
                        database = Room.databaseBuilder(
                            context.applicationContext,
                            AppDatabase::class.java,
                            "AndroidPSRemote"
                        )
                            .build()
                    }
                }
            }
            return database!!
        }
    }
}

class Converters {
    @TypeConverter
    fun macFromValue(v: Long) = MacAddress(v)

    @TypeConverter
    fun macToValue(addr: MacAddress) = addr.value

    @TypeConverter
    fun targetFromValue(v: Int) = Target.fromValue(v)

    @TypeConverter
    fun targetToValue(target: Target) = target.value
}