// SPDX-License-Identifier: LicenseRef-AGPL-3.0-only-OpenSSL

package co.vulcanlabs.psremote.di

import android.app.Application
import co.vulcanlabs.psremote.BuildConfig
import co.vulcanlabs.psremote.data.DeviceRepository
import co.vulcanlabs.psremote.data.DeviceRepositoryImpl
import co.vulcanlabs.psremote.data.UserRepository
import co.vulcanlabs.psremote.data.UserRepositoryImpl
import co.vulcanlabs.psremote.data.local.AppDatabase
import co.vulcanlabs.psremote.data.network.BaseNetwork
import co.vulcanlabs.psremote.data.network.service.PsnLoginService
import co.vulcanlabs.psremote.utils.DefaultErrorQuitMessageFactoryImpl
import co.vulcanlabs.psremote.utils.ErrorQuitMessageFactory
import co.vulcanlabs.psremote.utils.bodyToString
import dagger.Binds
import dagger.Module
import dagger.Provides
import dagger.hilt.InstallIn
import dagger.hilt.android.components.ActivityComponent
import dagger.hilt.components.SingletonComponent
import okhttp3.*
import java.net.URLDecoder
import javax.inject.Singleton

private const val USER_NAME = "ba495a24-818c-472b-b12d-ff231c1b5745"
private const val PASSWORD = "mvaiZkRsAsI1IBkY"
private const val BASE_URL = "https://auth.api.sonyentertainmentnetwork.com/"

@Module
@InstallIn(SingletonComponent::class)
abstract class ApplicationModule {

    companion object {
        @Provides
        @Singleton
        fun provideAppDatabase(app: Application): AppDatabase = AppDatabase.getInstance(app)

        @Provides
        @Singleton
        fun provideBaseNetwork(app: Application): BaseNetwork {
            val authToken =
                Credentials.basic(USER_NAME, PASSWORD)

            val authenticationInterceptor: (Interceptor.Chain) -> Response = { chain ->
                val original: Request = chain.request()
                val builder: Request.Builder = original.newBuilder()
                    .header("Authorization", authToken)
                chain.proceed(builder.build())
            }

            val decodedRedirectUriInterceptor: (Interceptor.Chain) -> Response = { chain ->
                val original: Request = chain.request()
                val builder: Request.Builder = original.newBuilder()
                val originalBody = original.body.bodyToString()
                if (originalBody?.contains("redirect_uri") == true) {
                    val newBody = URLDecoder.decode(originalBody)
                    builder.post(RequestBody.create(original.body!!.contentType(), newBody))
                }

                val request: Request = builder.build()
                chain.proceed(request)
            }

            val interceptors = listOf(authenticationInterceptor, decodedRedirectUriInterceptor)
            return BaseNetwork(
                app, BASE_URL, 20, false, BuildConfig.DEBUG, interceptors
            )
        }


        @Provides
        @Singleton
        fun providePsnLoginService(baseNetwork: BaseNetwork): PsnLoginService =
            baseNetwork.getService()
    }

    @Binds
    abstract fun bindUserRepository(impl: UserRepositoryImpl): UserRepository

    @Binds
    abstract fun bindDeviceRepository(impl: DeviceRepositoryImpl): DeviceRepository

}

@Module
@InstallIn(ActivityComponent::class)
abstract class ActivityModule {
    @Binds
    abstract fun bindQuitMessageFactory(impl: DefaultErrorQuitMessageFactoryImpl): ErrorQuitMessageFactory
}