// SPDX-License-Identifier: LicenseRef-AGPL-3.0-only-OpenSSL

package co.vulcanlabs.psremote.ui.alertdialog

import android.os.Bundle
import androidx.core.widget.doAfterTextChanged
import androidx.fragment.app.activityViewModels
import co.vulcanlabs.psremote.databinding.FragmentPinRequestBinding
import co.vulcanlabs.psremote.ui.base.BaseDialogFragment
import co.vulcanlabs.psremote.ui.play.PlayViewModel
import dagger.hilt.android.AndroidEntryPoint

@AndroidEntryPoint
class PinRequestDialogFragment :
    BaseDialogFragment<FragmentPinRequestBinding>(FragmentPinRequestBinding::class.java), DialogInPlayScreen {

    companion object {
        const val TAG = "PinRequestDialogFragment"
        fun newInstance() = PinRequestDialogFragment()
    }

    private val playViewModel: PlayViewModel by activityViewModels()

    override fun setupView(savedInstanceState: Bundle?) {
        isCancelable = false

        viewbinding?.run {

            pinInputLayout.editText?.doAfterTextChanged {
                btnReconnect.isEnabled = !it?.toString().isNullOrBlank()
            }

            txtQuit.setOnClickListener {
                dismissAllowingStateLoss()
                playViewModel.disconnect()
            }

            btnReconnect.setOnClickListener {
                val pinStr = pinInputLayout.editText?.text?.toString()?.trim().orEmpty()
                if (pinStr.isNotBlank()) {
                    playViewModel.setPin(pinStr)
                }
                dismissAllowingStateLoss()
            }
        }
    }

    override fun onOrientationChange() {
        activity?.supportFragmentManager?.run {
            beginTransaction()
                .remove(this@PinRequestDialogFragment)
                .commitNowAllowingStateLoss()

            newInstance()
                .showNow(this, TAG)
        }

    }
}