// SPDX-License-Identifier: LicenseRef-AGPL-3.0-only-OpenSSL

package co.vulcanlabs.psremote.ui.alertdialog

import android.os.Bundle
import androidx.core.os.bundleOf
import androidx.fragment.app.activityViewModels
import co.vulcanlabs.psremote.databinding.FragmentPlayInterruptBinding
import co.vulcanlabs.psremote.ui.base.BaseDialogFragment
import co.vulcanlabs.psremote.ui.play.PlayViewModel
import dagger.hilt.android.AndroidEntryPoint

@AndroidEntryPoint
class PlayInterruptDialogFragment : BaseDialogFragment<FragmentPlayInterruptBinding>(FragmentPlayInterruptBinding::class.java), DialogInPlayScreen {

    private val playViewModel: PlayViewModel by activityViewModels()

    companion object {
        const val ARG_CONTENT = "content"
        const val TAG = "PlayInterruptDialogFragment"

        fun newInstance(content: String) = PlayInterruptDialogFragment().apply {
            arguments = bundleOf(ARG_CONTENT to content)
        }
    }

    override fun setupView(savedInstanceState: Bundle?) {
        isCancelable = false

        viewbinding?.run {
            txtContent.text = arguments?.getString(ARG_CONTENT).orEmpty()

            txtQuit.setOnClickListener {
                dismissAllowingStateLoss()
                playViewModel.disconnect()
            }

            btnReconnect.setOnClickListener {
                dismissAllowingStateLoss()
                playViewModel.reconnect()
            }
        }

    }

    override fun onOrientationChange() {
        activity?.supportFragmentManager?.run {
            beginTransaction()
                .remove(this@PlayInterruptDialogFragment)
                .commitNowAllowingStateLoss()
            newInstance(
                arguments?.getString(
                    ARG_CONTENT,
                    ""
                ).orEmpty()
            )
                .showNow(this, TAG)
        }
    }
}