// SPDX-License-Identifier: LicenseRef-AGPL-3.0-only-OpenSSL

package co.vulcanlabs.psremote.ui.base

import android.app.Dialog
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.viewbinding.ViewBinding
import co.vulcanlabs.psremote.eventbus.RxBus
import co.vulcanlabs.psremote.eventbus.RxBusListener
import co.vulcanlabs.psremote.utils.getViewBinding
import com.google.android.material.bottomsheet.BottomSheetBehavior
import com.google.android.material.bottomsheet.BottomSheetDialog
import com.google.android.material.bottomsheet.BottomSheetDialogFragment


abstract class BaseBottomSheetDialogFragment<T : ViewBinding>(private val viewBindingClass: Class<T>) :
    BottomSheetDialogFragment(), RxBusListener {
    var onFinishLoading: (() -> Unit)? = null

    protected lateinit var viewbinding: T
        private set


    private val uuid by lazy { RxBus.INSTANCE.getNewUUID() }
    override fun getViewUUID() = uuid


    @Suppress("DEPRECATION")
    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?
    ): View? = getViewBinding(inflater, viewBindingClass)
        .apply { viewbinding = this }
        .root

    abstract fun setupView(savedInstanceState: Bundle?)

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        try {
            setupView(savedInstanceState)
        } catch (e: Exception) {
        }
    }

    override fun onCreateDialog(savedInstanceState: Bundle?): Dialog {
        return super.onCreateDialog(savedInstanceState).apply {
            setOnShowListener {
                (this as? BottomSheetDialog)?.behavior?.state = BottomSheetBehavior.STATE_EXPANDED
            }
        }
    }
}