// SPDX-License-Identifier: LicenseRef-AGPL-3.0-only-OpenSSL

package co.vulcanlabs.psremote.ui.base

import android.content.res.Configuration
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.annotation.CallSuper
import androidx.core.content.ContextCompat
import androidx.fragment.app.DialogFragment
import androidx.viewbinding.ViewBinding
import co.vulcanlabs.psremote.R
import co.vulcanlabs.psremote.eventbus.RxBus
import co.vulcanlabs.psremote.eventbus.RxBusListener
import co.vulcanlabs.psremote.utils.dpToPx
import co.vulcanlabs.psremote.utils.getViewBinding

abstract class BaseDialogFragment<T : ViewBinding>(private val viewBindingClass: Class<T>) : DialogFragment(),
    RxBusListener {

    var onFinishLoading: (() -> Unit)? = null

    open val isFullScreen: Boolean = false

    var viewbinding: T? = null


    private val uuid by lazy { RxBus.INSTANCE.getNewUUID() }

    override fun getViewUUID() = uuid

    abstract fun setupView(savedInstanceState: Bundle?)

    @CallSuper
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        if (isFullScreen) {
            setStyle(STYLE_NORMAL, android.R.style.Theme_Translucent_NoTitleBar)
        }
    }

    @Suppress("DEPRECATION")
    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?
    ): View? {

        return getViewBinding(inflater, viewBindingClass)
            .apply { viewbinding = this }
            .root

    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        try {
            setupView(savedInstanceState)
            onFinishLoading?.invoke()
        } catch (e: Exception) {
        }
    }

    override fun onDestroyView() {
        super.onDestroyView()
        viewbinding = null
    }

    @CallSuper
    override fun onStart() {
        super.onStart()
        val width = requireContext().dpToPx(
            (if (resources.configuration.orientation == Configuration.ORIENTATION_LANDSCAPE) {
                resources.configuration.screenWidthDp * 0.7
            } else {
                resources.configuration.screenWidthDp * 0.85
            }).toInt()
        )
        dialog?.window?.setLayout(
            width.toInt(),
            ViewGroup.LayoutParams.WRAP_CONTENT
        )
        dialog?.window?.setBackgroundDrawable(
            ContextCompat.getDrawable(
                requireContext(),
                R.drawable.bg_alert_dialog
            )
        )
    }
}