// SPDX-License-Identifier: LicenseRef-AGPL-3.0-only-OpenSSL

package co.vulcanlabs.psremote.ui.base

import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.annotation.CallSuper
import androidx.fragment.app.Fragment
import androidx.viewbinding.ViewBinding
import co.vulcanlabs.psremote.eventbus.RxBus
import co.vulcanlabs.psremote.eventbus.RxBusListener
import co.vulcanlabs.psremote.utils.getViewBinding

abstract class BaseFragment<T : ViewBinding>(val clazz: Class<T>) : Fragment(),
    RxBusListener {
    var viewbinding: T? = null

    private val uuid by lazy { RxBus.INSTANCE.getNewUUID() }

    override fun getViewUUID() = uuid

    @Suppress("DEPRECATION")
    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?
    ): View? {

        return getViewBinding(inflater, clazz)
            .apply { viewbinding = this }
            .root

    }

    abstract fun setupView(savedInstanceState: Bundle?)

    @CallSuper
    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        try {
            setupView(savedInstanceState)
        } catch (e: Exception) {
        }
    }

    override fun onDestroyView() {
        viewbinding = null
        super.onDestroyView()
    }

    override fun onDestroy() {
        super.onDestroy()
        RxBus.INSTANCE.unRegister(this)
    }
}