// SPDX-License-Identifier: LicenseRef-AGPL-3.0-only-OpenSSL

package co.vulcanlabs.psremote.ui.base

import android.app.Application
import androidx.lifecycle.ViewModel
import co.vulcanlabs.psremote.eventbus.RxBus
import co.vulcanlabs.psremote.eventbus.RxBusListener
import io.reactivex.rxjava3.disposables.CompositeDisposable
import io.reactivex.rxjava3.disposables.Disposable

open class BaseViewModel(val app: Application) : ViewModel(), RxBusListener {
    private var compositeDisposable: CompositeDisposable? = null

    private val uuid by lazy { RxBus.INSTANCE.getNewUUID() }
    override fun getViewUUID() = uuid

    fun addDisposable(disposable: Disposable) {
        if (compositeDisposable == null || compositeDisposable?.isDisposed == true) {
            compositeDisposable = CompositeDisposable()
        }
        compositeDisposable?.add(disposable)
    }

    override fun onCleared() {
        compositeDisposable?.clear()
        RxBus.INSTANCE.unRegister(this)
        super.onCleared()
    }
}