// SPDX-License-Identifier: LicenseRef-AGPL-3.0-only-OpenSSL

package co.vulcanlabs.psremote.ui.discovery

import androidx.compose.foundation.Image
import androidx.compose.foundation.background
import androidx.compose.foundation.clickable
import androidx.compose.foundation.layout.*
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.shape.RoundedCornerShape
import androidx.compose.material.Text
import androidx.compose.runtime.Composable
import androidx.compose.runtime.rxjava3.subscribeAsState
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.draw.clip
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.res.painterResource
import androidx.compose.ui.tooling.preview.Preview
import androidx.compose.ui.unit.dp
import androidx.compose.ui.unit.sp
import androidx.lifecycle.viewmodel.compose.viewModel
import co.vulcanlabs.psremote.chiaki.DiscoveryHost
import co.vulcanlabs.psremote.common.DiscoveredDisplayHost
import co.vulcanlabs.psremote.common.DisplayHost
import co.vulcanlabs.psremote.R


@Composable
fun DeviceList(modifier: Modifier, onDeviceClick: (DisplayHost) -> Unit, devices: List<DisplayHost>) {
    LazyColumn(modifier = modifier, contentPadding = PaddingValues(top = 10.dp)) {
        items(devices.size, { devices[it].id.orEmpty() }) {
            val item = devices[it]
            DeviceItem(item, onDeviceClick)
        }
    }
}

@Composable
fun DeviceItem(device: DisplayHost, onDeviceClick: (DisplayHost) -> Unit) {
    Box(
        Modifier
            .padding(
                top = 15.dp,
                start = 16.dp,
                end = 16.dp
            )
            .height(120.dp)
            .fillMaxWidth()
            .clip(RoundedCornerShape(12.dp))
            .clickable(onClick = { onDeviceClick(device) })
            .background(Color(0xff, 0xff, 0xff, 0x33))
            .padding(20.dp), contentAlignment = Alignment.CenterStart
    ) {
        Image(
            painter = painterResource(id = if (device.isPS5) R.drawable.ic_ps5 else R.drawable.ic_ps4),
            contentDescription = ""
        )

        Column(
            modifier = Modifier.padding(start = 100.dp),
            horizontalAlignment = Alignment.CenterHorizontally
        ) {
            Text(text = device.name.orEmpty(), color = Color.White, fontSize = 20.sp)
            if (device.isRegistered) {
                Text(
                    text = "Registered",
                    fontSize = 16.sp,
                    color = Color(0xff, 0xff, 0xff, 0x8c)
                )
            }
        }
    }

}

@Preview
@Composable
fun PreviewDevice() {
    val device = DiscoveredDisplayHost(
        null,
        DiscoveryHost(
            DiscoveryHost.State.READY,
            0U,
            "192.168.1.1",
            null,
            null,
            "Name",
            null,
            null,
            null,
            null
        )
    )
    DeviceItem(device = device, onDeviceClick = { })
}