// SPDX-License-Identifier: LicenseRef-AGPL-3.0-only-OpenSSL

package co.vulcanlabs.psremote.ui.discovery

import androidx.compose.animation.core.Animatable
import androidx.compose.animation.core.AnimationEndReason
import androidx.compose.animation.core.tween
import androidx.compose.foundation.Image
import androidx.compose.foundation.clickable
import androidx.compose.foundation.layout.*
import androidx.compose.foundation.shape.CircleShape
import androidx.compose.material.Text
import androidx.compose.runtime.Composable
import androidx.compose.runtime.LaunchedEffect
import androidx.compose.runtime.collectAsState
import androidx.compose.runtime.remember
import androidx.compose.ui.Modifier
import androidx.compose.ui.draw.clip
import androidx.compose.ui.draw.rotate
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.res.painterResource
import androidx.compose.ui.text.style.TextAlign
import androidx.compose.ui.unit.dp
import androidx.compose.ui.unit.sp
import androidx.constraintlayout.compose.ConstraintLayout
import androidx.lifecycle.viewmodel.compose.viewModel
import co.vulcanlabs.psremote.R

@Composable
fun DiscoveryTopBar(
    modifier: Modifier,
    onBackClick: () -> Unit,
    onLogoutClick: () -> Unit
) {
    val viewModel: DiscoveryViewModel = viewModel()
    val isRefreshing = viewModel.isRefreshingState.collectAsState()
    ConstraintLayout(
        modifier
    ) {

        val (topLeftBtn, title, refreshBtn) = createRefs()

        Text(
            text = "Your PS devices",
            color = Color.White,
            fontSize = 22.sp,
            modifier = Modifier.constrainAs(title) {
                start.linkTo(parent.start)
                end.linkTo(parent.end)
            },
            textAlign = TextAlign.Center
        )
        val asDeviceManagement = viewModel.asDeviceManagement.collectAsState()
        Image(painter = painterResource(id = if (asDeviceManagement.value) R.drawable.ic_back else R.drawable.ic_logout),
            contentDescription = "",
            modifier = Modifier
                .constrainAs(topLeftBtn) {
                    start.linkTo(parent.start, margin = 16.dp)
                    top.linkTo(parent.top)
                    bottom.linkTo(parent.bottom)
                }
                .clip(CircleShape)
                .clickable(
                    onClick = if (asDeviceManagement.value) {
                        onBackClick
                    } else {
                        onLogoutClick
                    }
                )
                .padding(4.dp)
        )

        val rotationAnimatable = remember { Animatable(0f) }
        LaunchedEffect(isRefreshing.value, rotationAnimatable.isRunning) {
            rotationAnimatable.stop()
            val result = if (isRefreshing.value) {
                rotationAnimatable.animateTo(
                    ((rotationAnimatable.value + 2880f) / 360f).toInt() * 360f,
                    tween(2500)
                )
            } else {
                rotationAnimatable.animateTo(
                    ((rotationAnimatable.value + 720f) / 360f).toInt() * 360f,
                    tween(2500)
                )
            }
            if (result.endReason == AnimationEndReason.Finished && isRefreshing.value) {
                viewModel.doneRefreshing()
            }
        }

        Image(
            painter = painterResource(id = R.drawable.ic_refresh),
            contentDescription = "",
            modifier = Modifier
                .size(32.dp)
                .constrainAs(refreshBtn) {
                    start.linkTo(title.end, margin = 13.dp)
                    top.linkTo(title.top)
                    bottom.linkTo(title.bottom)
                }
                .rotate(rotationAnimatable.value)
                .clip(CircleShape)
                .clickable(
                    enabled = !isRefreshing.value,
                    onClick = { viewModel.refreshDiscovery() })
                .padding(4.dp)
        )
    }
}