// SPDX-License-Identifier: LicenseRef-AGPL-3.0-only-OpenSSL

package co.vulcanlabs.psremote.ui.play

import android.content.Context
import android.util.AttributeSet
import android.view.Gravity
import android.widget.FrameLayout
import kotlin.math.min

// see ExoPlayer's AspectRatioFrameLayout
class AspectRatioFrameLayout @JvmOverloads constructor(
    context: Context,
    attrs: AttributeSet? = null
) : FrameLayout(context, attrs) {

    private var childAspectRatio = 0f

    private var childGravity: Int = Gravity.TOP

    fun setProperties(childAspectRatio: Float, childGravity: Int) {
        this.childAspectRatio = childAspectRatio
        this.childGravity = childGravity
        applyRatio()
    }

    private fun applyRatio() {
        post {
            val child = getChildAt(0) ?: return@post
            val viewWidth = width
            val viewHeight = height

            val childWidth = 1f
            val childHeight: Float = childWidth / childAspectRatio

            val scale = min(viewWidth / childWidth, viewHeight / childHeight)

            val childLayoutParams = child.layoutParams as? LayoutParams ?: return@post
            with(childLayoutParams) {
                this.width = (childWidth * scale).toInt()
                this.height = (childHeight * scale).toInt()
                this.gravity = childGravity
                child.layoutParams = this
            }

        }
    }

    override fun onSizeChanged(w: Int, h: Int, oldw: Int, oldh: Int) {
        super.onSizeChanged(w, h, oldw, oldh)
        applyRatio()
    }
}