// SPDX-License-Identifier: LicenseRef-AGPL-3.0-only-OpenSSL

package co.vulcanlabs.psremote.ui.play

import android.os.Bundle
import androidx.core.view.isVisible
import androidx.fragment.app.activityViewModels
import androidx.lifecycle.lifecycleScope
import co.vulcanlabs.psremote.databinding.FragmentPlaySettingBinding
import co.vulcanlabs.psremote.ui.base.BaseBottomSheetDialogFragment
import co.vulcanlabs.psremote.utils.collectWhenStarted

class PlaySettingFragment :
    BaseBottomSheetDialogFragment<FragmentPlaySettingBinding>(FragmentPlaySettingBinding::class.java) {

    private val playViewModel: PlayViewModel by activityViewModels()

    override fun setupView(savedInstanceState: Bundle?) {
        viewbinding.imgClose.setOnClickListener {
            dismissAllowingStateLoss()
        }
        viewbinding.imgAutoRotate.setOnClickListener {
            playViewModel.setLockOrientationMode(PlayViewModel.LockOrientationMode.AUTO_ROTATE)
        }
        viewbinding.imgLockLandscape.setOnClickListener {
            playViewModel.setLockOrientationMode(PlayViewModel.LockOrientationMode.LOCK_LANDSCAPE)

        }
        viewbinding.imgLockPortrait.setOnClickListener {
            playViewModel.setLockOrientationMode(PlayViewModel.LockOrientationMode.LOCK_PORTRAIT)
        }

        viewbinding.chkAlwaysShowController.setOnCheckedChangeListener { _, isChecked ->
            playViewModel.setAlwaysShowController(isChecked)
        }

        viewbinding.disconnectLayout.setOnClickListener {
            playViewModel.disconnect()
        }

        playViewModel.lockOrientationMode.collectWhenStarted(viewLifecycleOwner.lifecycleScope) {
            viewbinding.imgAutoRotate.isSelected =
                it == PlayViewModel.LockOrientationMode.AUTO_ROTATE
            viewbinding.imgLockPortrait.isSelected =
                it == PlayViewModel.LockOrientationMode.LOCK_PORTRAIT
            viewbinding.imgLockLandscape.isSelected =
                it == PlayViewModel.LockOrientationMode.LOCK_LANDSCAPE
        }

        playViewModel.alwaysShowController.collectWhenStarted(viewLifecycleOwner.lifecycleScope) {
            if (it != null && viewbinding.chkAlwaysShowController.isChecked != it) {
                viewbinding.chkAlwaysShowController.isChecked = it
            }
        }

        viewbinding.alwaysShowControllerLayout.isVisible = playViewModel.mode != PlayViewModel.Mode.Gamepad
        viewbinding.orientationModeLayout.isVisible = playViewModel.mode != PlayViewModel.Mode.Gamepad
    }

    companion object {
        fun newInstance() = PlaySettingFragment()

        const val TAG = "PlaySettingFragment"
    }
}