// SPDX-License-Identifier: LicenseRef-AGPL-3.0-only-OpenSSL

package co.vulcanlabs.psremote.ui.profile

import android.content.Intent
import android.os.Bundle
import androidx.activity.viewModels
import androidx.lifecycle.lifecycleScope
import co.vulcanlabs.psremote.R
import co.vulcanlabs.psremote.data.Result
import co.vulcanlabs.psremote.databinding.ActivityProfileSettingBinding
import co.vulcanlabs.psremote.management.DeviceManager
import co.vulcanlabs.psremote.ui.base.BaseActivity
import co.vulcanlabs.psremote.ui.alertdialog.LogoutAlertDialogFragment
import co.vulcanlabs.psremote.ui.splash.SplashActivity
import co.vulcanlabs.psremote.utils.collectLatestWhenStarted
import co.vulcanlabs.psremote.utils.showDialog
import co.vulcanlabs.psremote.utils.showToast
import co.vulcanlabs.psremote.utils.startActivityView
import dagger.hilt.android.AndroidEntryPoint

@AndroidEntryPoint
class ProfileSettingActivity :
    BaseActivity<ActivityProfileSettingBinding>(ActivityProfileSettingBinding::class.java), LogoutAlertDialogFragment.OnLogoutConfirmListener {

    private val viewModel: ProfileSettingViewModel by viewModels()

    override fun setupView(savedInstanceState: Bundle?) {
        viewbinding.layoutLogout.setOnClickListener {
            showDialog(LogoutAlertDialogFragment.newInstance(), LogoutAlertDialogFragment.TAG)
        }

        viewbinding.imgBack.setOnClickListener {
            finish()
        }


        viewModel.currentUserFlow.collectLatestWhenStarted(lifecycleScope) {
            if (it == null) {
                return@collectLatestWhenStarted
            }
            with(viewbinding) {
                txtPsnId.text = getString(R.string.psn_id_format, it.onlineId)
                txtPsnAccountId.text = getString(R.string.psn_account_id_format, it.userId)
            }
        }

        viewModel.currentDeviceNameFlow.collectLatestWhenStarted(lifecycleScope) {
            viewbinding.txtPsNickName.text = getString(R.string.ps_nickname_format, it)
        }

        viewModel.logOutEvent.collectLatestWhenStarted(lifecycleScope) {
            if (it is Result.Success) {
                DeviceManager.setCurrentDevice(null)
                startActivityView<SplashActivity> {
                    flags = Intent.FLAG_ACTIVITY_NEW_TASK or Intent.FLAG_ACTIVITY_CLEAR_TASK
                }
            } else if (it is Result.Failure) {
                it.exception.message.orEmpty().showToast(false)
            }
        }
    }

    override fun onLogoutConfirm() {
        viewModel.logOut()
    }
}