// SPDX-License-Identifier: LicenseRef-AGPL-3.0-only-OpenSSL

package co.vulcanlabs.psremote.ui.profile

import android.app.Application
import androidx.lifecycle.viewModelScope
import co.vulcanlabs.psremote.common.UserAccount
import co.vulcanlabs.psremote.data.DeviceRepository
import co.vulcanlabs.psremote.data.Result
import co.vulcanlabs.psremote.data.UserRepository
import co.vulcanlabs.psremote.management.DeviceManager
import co.vulcanlabs.psremote.ui.base.BaseViewModel
import dagger.hilt.android.lifecycle.HiltViewModel
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.flow.MutableSharedFlow
import kotlinx.coroutines.flow.MutableStateFlow
import kotlinx.coroutines.flow.asSharedFlow
import kotlinx.coroutines.flow.asStateFlow
import kotlinx.coroutines.launch
import javax.inject.Inject

@HiltViewModel
class ProfileSettingViewModel @Inject constructor(
    app: Application,
    private val userRepository: UserRepository,
    private val deviceRepository: DeviceRepository
) : BaseViewModel(app) {

    private val _currentUserFlow = MutableStateFlow<UserAccount?>(null)
    val currentUserFlow = _currentUserFlow.asStateFlow()

    private val _currentDeviceNameFlow = MutableStateFlow("")
    val currentDeviceNameFlow = _currentDeviceNameFlow.asStateFlow()

    private val _logOutEvent = MutableSharedFlow<Result<Unit>>()
    val logOutEvent = _logOutEvent.asSharedFlow()

    init {
        viewModelScope.launch(Dispatchers.IO) {
            _currentUserFlow.value = userRepository.getCurrentUser()
        }

        viewModelScope.launch(Dispatchers.IO) {
            _currentDeviceNameFlow.value = DeviceManager.getCurrentDevice()?.name.orEmpty()
        }
    }

    fun logOut() = viewModelScope.launch {
        when (val logOutResult = userRepository.logOut()) {
            is Result.Failure -> _logOutEvent.emit(logOutResult)
            is Result.Success -> _logOutEvent.emit(deviceRepository.clearAllDevices())
        }
    }
}