// SPDX-License-Identifier: LicenseRef-AGPL-3.0-only-OpenSSL

package co.vulcanlabs.psremote.ui.psnlogin

import android.content.Intent
import android.os.Bundle
import co.vulcanlabs.psremote.R
import co.vulcanlabs.psremote.databinding.ActivityPsnLoginBinding
import co.vulcanlabs.psremote.ui.base.BaseActivity
import co.vulcanlabs.psremote.ui.discovery.DiscoveryActivity
import co.vulcanlabs.psremote.utils.showToast
import co.vulcanlabs.psremote.utils.startActivityView
import dagger.hilt.android.AndroidEntryPoint

@AndroidEntryPoint
class PsnLoginActivity :
    BaseActivity<ActivityPsnLoginBinding>(ActivityPsnLoginBinding::class.java),
    PsnLoginFragment.OnLoginPsnResultListener {


    override fun setupView(savedInstanceState: Bundle?) {
        supportFragmentManager.beginTransaction()
            .replace(
                R.id.container, PsnLoginFragment.newInstance()
            )
            .commit()
    }

    override fun onPsnLoginSuccess() {
        startActivityView<DiscoveryActivity> {
            putExtra(DiscoveryActivity.EXTRA_SHOW_LOGIN_SUCCESS, true)
            flags = Intent.FLAG_ACTIVITY_CLEAR_TASK or Intent.FLAG_ACTIVITY_NEW_TASK
        }
    }

    override fun onPsnLoginFailure(cause: Exception) {
        val errorMessage =
            cause.message.takeIf { !it.isNullOrEmpty() } ?: cause.javaClass.simpleName
        errorMessage.showToast(false)
        finish()
    }
}