// SPDX-License-Identifier: LicenseRef-AGPL-3.0-only-OpenSSL

package co.vulcanlabs.psremote.ui.psnlogin

import android.annotation.SuppressLint
import android.os.Bundle
import android.webkit.WebResourceRequest
import android.webkit.WebView
import android.webkit.WebViewClient
import androidx.fragment.app.viewModels
import androidx.lifecycle.lifecycleScope
import co.vulcanlabs.psremote.databinding.FragmentPsnLoginBinding
import co.vulcanlabs.psremote.ui.base.BaseFragment
import co.vulcanlabs.psremote.utils.collectLatestWhenStarted
import co.vulcanlabs.psremote.utils.hideAppLoading
import co.vulcanlabs.psremote.utils.showAppLoading
import dagger.hilt.android.AndroidEntryPoint

@AndroidEntryPoint
class PsnLoginFragment :
    BaseFragment<FragmentPsnLoginBinding>(FragmentPsnLoginBinding::class.java) {

    private val viewModel: PsnLoginViewModel by viewModels()

    @SuppressLint("SetJavaScriptEnabled")
    override fun setupView(savedInstanceState: Bundle?) {

        viewModel.loginStateFlow.collectLatestWhenStarted(viewLifecycleOwner.lifecycleScope) {
            if (it == PsnLoginViewModel.LoginState.Loading) {
                showAppLoading()
            } else {
                hideAppLoading()
            }
            when (it) {
                PsnLoginViewModel.LoginState.Success -> {
                    (activity as? OnLoginPsnResultListener)?.onPsnLoginSuccess()
                }
                is PsnLoginViewModel.LoginState.Failure -> {
                    (activity as? OnLoginPsnResultListener)?.onPsnLoginFailure(it.cause)
                }
                else -> {}
            }
        }

        viewbinding?.webView?.settings?.run {
            javaScriptEnabled = true
        }
        viewbinding?.webView?.run {
            webViewClient = object : WebViewClient() {
                override fun shouldOverrideUrlLoading(
                    view: WebView?,
                    request: WebResourceRequest?
                ): Boolean {
                    request?.url?.getQueryParameter("code")?.takeIf { it.isNotEmpty() }?.run {
                        viewModel.loginPsn(this)
                    }
                    return false
                }
            }

            loadUrl(PSN_URL)
        }
        viewbinding?.imgBack?.setOnClickListener {
            requireActivity().finish()
        }
    }

    interface OnLoginPsnResultListener {
        fun onPsnLoginSuccess()
        fun onPsnLoginFailure(cause: Exception)
    }

    companion object {
        fun newInstance() = PsnLoginFragment()

        private const val PSN_URL = "https://auth.api.sonyentertainmentnetwork.com/2.0/" +
                "oauth/authorize?service_entity=urn:service-entity:psn&response_type=" +
                "code&client_id=ba495a24-818c-472b-b12d-ff231c1b5745&redirect_uri=https://remoteplay.dl.playstation.net/remoteplay/redirect" +
                "&scope=psn:clientapp&request_locale=en_US&ui=pr&service_logo=ps&layout_type=popup&smcid" +
                "=remoteplay&prompt=always&PlatformPrivacyWs1=minimal&"

    }

}