// SPDX-License-Identifier: LicenseRef-AGPL-3.0-only-OpenSSL

package co.vulcanlabs.psremote.ui.psnlogin

import android.app.Application
import androidx.lifecycle.viewModelScope
import co.vulcanlabs.psremote.data.Result
import co.vulcanlabs.psremote.data.UserRepository
import co.vulcanlabs.psremote.ui.base.BaseViewModel
import dagger.hilt.android.lifecycle.HiltViewModel
import kotlinx.coroutines.flow.MutableStateFlow
import kotlinx.coroutines.flow.asSharedFlow
import kotlinx.coroutines.launch
import javax.inject.Inject

@HiltViewModel
class PsnLoginViewModel @Inject constructor(
    app: Application,
    private val repo: UserRepository
) : BaseViewModel(app) {

    private val _loginStateFlow = MutableStateFlow<LoginState>(LoginState.Initial)
    val loginStateFlow = _loginStateFlow.asSharedFlow()

    fun loginPsn(code: String) {
        viewModelScope.launch {
            _loginStateFlow.value = LoginState.Loading
            when (val psnIdResult = repo.loginPsn(code)) {
                is Result.Success -> _loginStateFlow.value = LoginState.Success
                is Result.Failure -> _loginStateFlow.value = LoginState.Failure(psnIdResult.exception)
            }
        }
    }

    sealed class LoginState {

        object Initial : LoginState()

        object Loading : LoginState()

        object Success : LoginState()

        data class Failure(val cause: Exception) : LoginState()
    }
}