// SPDX-License-Identifier: LicenseRef-AGPL-3.0-only-OpenSSL

package co.vulcanlabs.psremote.ui.registerdevice.selectfirmware

import android.content.DialogInterface
import android.os.Bundle
import android.view.WindowManager
import android.widget.FrameLayout
import androidx.core.os.bundleOf
import androidx.fragment.app.viewModels
import androidx.lifecycle.lifecycleScope
import co.vulcanlabs.psremote.R
import co.vulcanlabs.psremote.common.ConsoleVersion
import co.vulcanlabs.psremote.databinding.FragmentFirmwareSelectBinding
import co.vulcanlabs.psremote.ui.base.BaseBottomSheetDialogFragment
import co.vulcanlabs.psremote.utils.collectLatestWhenStarted
import com.google.android.material.bottomsheet.BottomSheetDialog
import dagger.hilt.android.AndroidEntryPoint

@AndroidEntryPoint
class FirmwareSelectDialogFragment :
    BaseBottomSheetDialogFragment<FragmentFirmwareSelectBinding>(FragmentFirmwareSelectBinding::class.java) {

    companion object {
        private const val ARG_SELECTED_CONSOLE_VERSION = "ARG_SELECTED_CONSOLE_VERSION"

        const val TAG = "FirmwareSelectDialogFragment"

        fun newInstance(selectedConsoleVersion: ConsoleVersion?) =
            FirmwareSelectDialogFragment().apply {
                arguments = bundleOf(ARG_SELECTED_CONSOLE_VERSION to selectedConsoleVersion)
            }
    }

    private val firmwareSelectViewModel: FirmwareSelectViewModel by viewModels()

    lateinit var adapter: SystemSoftwareAdapter

    override fun setupView(savedInstanceState: Bundle?) {
        setUpBinding(viewbinding)

        (arguments?.getSerializable(ARG_SELECTED_CONSOLE_VERSION) as? ConsoleVersion)?.let {
            firmwareSelectViewModel.selectFirmware(it)
        }

        firmwareSelectViewModel.firmwares.collectLatestWhenStarted(viewLifecycleOwner.lifecycleScope) {
            if (!::adapter.isInitialized) {
                return@collectLatestWhenStarted
            }
            adapter.updateList(it)
            val selected = it.find { firmwareUiModel -> firmwareUiModel.isSelected }
            if (selected != null) {
                arguments?.putSerializable(ARG_SELECTED_CONSOLE_VERSION, selected.consoleVersion)
                (parentFragment as? OnDialogInteractionListener)?.onFirmwareSelected(selected.consoleVersion)
            }
        }
    }

    private fun setUpBinding(binding: FragmentFirmwareSelectBinding) {
        adapter = SystemSoftwareAdapter()
        adapter.setRecycleView(binding.rvFirmware)
        adapter.updateList(firmwareSelectViewModel.firmwares.value)
        adapter.onItemClick = { position, _ ->
            firmwareSelectViewModel.selectFirmwareAt(position)
        }

        viewbinding.outsideView?.setOnClickListener {
            dismissAllowingStateLoss()
        }
    }

    override fun onDismiss(dialog: DialogInterface) {
        super.onDismiss(dialog)
        (parentFragment as? OnDialogInteractionListener)?.onDialogDismiss()
    }

    override fun onStart() {
        super.onStart()
        (dialog as? BottomSheetDialog)?.run {
            window?.clearFlags(WindowManager.LayoutParams.FLAG_DIM_BEHIND)
            window?.findViewById<FrameLayout>(R.id.design_bottom_sheet)
                ?.setBackgroundResource(android.R.color.transparent)
        }
    }

    interface OnDialogInteractionListener {
        fun onFirmwareSelected(consoleVersion: ConsoleVersion)

        fun onDialogDismiss()
    }
}