// SPDX-License-Identifier: LicenseRef-AGPL-3.0-only-OpenSSL

package co.vulcanlabs.psremote.ui.registerdevice.selectfirmware

import android.app.Application
import co.vulcanlabs.psremote.common.ConsoleVersion
import co.vulcanlabs.psremote.ui.base.BaseViewModel
import dagger.hilt.android.lifecycle.HiltViewModel
import kotlinx.coroutines.flow.MutableStateFlow
import kotlinx.coroutines.flow.asStateFlow
import javax.inject.Inject

@HiltViewModel
class FirmwareSelectViewModel @Inject constructor(app: Application) : BaseViewModel(app) {

    private val _firmwares = MutableStateFlow(getFirmwares())

    val firmwares = _firmwares.asStateFlow()

    fun selectFirmwareAt(position: Int) {
        _firmwares.value = _firmwares.value.mapIndexed { index, firmwareUiModel ->
            firmwareUiModel.copy(isSelected = index == position)
        }
    }

    private fun getFirmwares(): List<FirmwareUiModel> = listOf(
        FirmwareUiModel(ConsoleVersion.PS5, false),
        FirmwareUiModel(ConsoleVersion.PS4_GE_8, true),
        FirmwareUiModel(ConsoleVersion.PS4_GE_7, false),
        FirmwareUiModel(ConsoleVersion.PS4_LT_7, false)
    )

    fun selectFirmware(version: ConsoleVersion) {
        firmwares.value.indexOfFirst { it.consoleVersion == version }.let {
            selectFirmwareAt(it)
        }
    }
}