// SPDX-License-Identifier: LicenseRef-AGPL-3.0-only-OpenSSL

package co.vulcanlabs.psremote.ui.touchcontrols

import android.content.Context
import android.util.AttributeSet
import android.view.LayoutInflater
import android.widget.FrameLayout
import co.vulcanlabs.psremote.databinding.LayoutFaceButtonViewBinding

class FaceButtonView @JvmOverloads constructor(
    context: Context, attrs: AttributeSet? = null, defStyleAttr: Int = 0
) : FrameLayout(context, attrs, defStyleAttr) {

    private val binding: LayoutFaceButtonViewBinding

    var buttonPressedCallback: ((Boolean, ChildType) -> Unit)? = null

    init {
        val inflater = LayoutInflater.from(context)
        binding = LayoutFaceButtonViewBinding.inflate(inflater, this, true)
    }

    override fun onFinishInflate() {
        super.onFinishInflate()
        binding.boxButtonView.buttonPressedCallback = {
            buttonPressedCallback?.invoke(it, ChildType.BOX)
        }
        binding.pyramidButtonView.buttonPressedCallback = {
            buttonPressedCallback?.invoke(it, ChildType.PYRAMID)
        }
        binding.moonButtonView.buttonPressedCallback = {
            buttonPressedCallback?.invoke(it, ChildType.MOON)
        }
        binding.crossButtonView.buttonPressedCallback = {
            buttonPressedCallback?.invoke(it, ChildType.CROSS)
        }
    }

    enum class ChildType {
        BOX, PYRAMID, MOON, CROSS
    }
}