// SPDX-License-Identifier: LicenseRef-AGPL-3.0-only-OpenSSL

package co.vulcanlabs.psremote.utils

import android.app.Application
import co.vulcanlabs.psremote.R
import co.vulcanlabs.psremote.chiaki.QuitReason
import javax.inject.Inject

interface ErrorQuitMessageFactory {
    fun create(quitReason: QuitReason): String
}

class DefaultErrorQuitMessageFactoryImpl @Inject constructor(private val app: Application) :
    ErrorQuitMessageFactory {
    override fun create(quitReason: QuitReason): String {
        val stringResId: Int = when (quitReason.value) {
            QuitReason.CHIAKI_QUIT_REASON_SESSION_REQUEST_UNKNOWN -> R.string.error_quit_message_session_request_unknown
            QuitReason.CHIAKI_QUIT_REASON_SESSION_REQUEST_CONNECTION_REFUSED -> R.string.error_quit_message_session_request_connection_refused
            QuitReason.CHIAKI_QUIT_REASON_SESSION_REQUEST_RP_IN_USE -> R.string.error_quit_message_session_request_rp_in_use
            QuitReason.CHIAKI_QUIT_REASON_SESSION_REQUEST_RP_CRASH -> R.string.error_quit_message_session_request_rp_crash
            QuitReason.CHIAKI_QUIT_REASON_SESSION_REQUEST_RP_VERSION_MISMATCH -> R.string.error_quit_message_session_request_rp_version_mismatch
            QuitReason.CHIAKI_QUIT_REASON_CTRL_UNKNOWN -> R.string.error_quit_message_ctrl_unknown
            QuitReason.CHIAKI_QUIT_REASON_CTRL_CONNECTION_REFUSED -> R.string.error_quit_message_ctrl_connection_refused
            QuitReason.CHIAKI_QUIT_REASON_CTRL_CONNECT_FAILED -> R.string.error_quit_message_ctrl_connection_refused
            QuitReason.CHIAKI_QUIT_REASON_STREAM_CONNECTION_UNKNOWN -> R.string.error_quit_message_stream_connection_unknown
            QuitReason.CHIAKI_QUIT_REASON_STREAM_CONNECTION_REMOTE_DISCONNECTED -> R.string.error_quit_message_stream_connection_remote_disconnected
            QuitReason.CHIAKI_QUIT_REASON_STREAM_CONNECTION_REMOTE_DISCONNECTED -> R.string.error_quit_message_stream_connection_remote_disconnected
            QuitReason.CHIAKI_QUIT_REASON_STOPPED -> R.string.error_quit_message_stream_connection_remote_stopped
            else -> R.string.error_quit_message_unknown
        }

        return app.getString(stringResId)
    }
}